import board
import busio
import adafruit_mpr121
import datetime

from micropython import const


class mpr121:

    def __init__(self,adapt=True):
        self.i2c = busio.I2C(board.SCL, board.SDA)

        self.MPR121_TOUCHTH_0       = const(0x41)
        self.MPR121_RELEASETH_0     = const(0x42)
        
        self.end = False

        # Get all 4 devices:
        self.mprs =[]        
        self.mprs.append(adafruit_mpr121.MPR121(self.i2c, address=0x5C))
        self.mprs.append(adafruit_mpr121.MPR121(self.i2c, address=0x5D))
        self.mprs.append(adafruit_mpr121.MPR121(self.i2c, address=0x5B))
        self.mprs.append(adafruit_mpr121.MPR121(self.i2c, address=0x5A))

        self.connectedPins = [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                             21,20,19,18,17,16,15,14,13,12,
                             24,25,26,27,28,29,30,31,32,33,
                             38,39,40,41,42,47,46,45,44,43]    #These are the physically connected Pins
        
        self.states = [0]*10*len(self.mprs)
        self.last_time_up = [datetime.datetime.now()]*10*len(self.mprs)
        
        if(adapt):
            self.adapt_thresholds()            

    def quit_loop(self):
        self.end = True
    
    def adapt_thresholds(self):
        #Change threshold values for the touch-detection
        mprCount = 0
        for mpr in self.mprs:
            mprCount = mprCount +1
            touchThr = int(9-mprCount)
            releaseThr = int(7-mprCount)
            
            print('MPR',mprCount,'threshold values: touch',touchThr,'release',releaseThr)
            
            for i in range(12):                   
                mpr._write_register_byte(self.MPR121_TOUCHTH_0 + 2*i, touchThr) # touch-threshold value, default is 10
                mpr._write_register_byte(self.MPR121_RELEASETH_0 + 2*i, releaseThr)  # release-threshold value, default is 5
    
    def map_pins(self,index):
      # Pins:
      #  0  9 21 12 24 33  38 47
      #  1  8 20 13 25 32  39 46
      #  2  7 19 14 26 31  40 45
      #  3  6 18 15 27 30  41 44
      #  4  5 17 16 28 29  42 43

      # Should-be Pins:
      #  0  9 10 19 20 29 30 39
      #  1  8 11 18 21 28 31 38
      #  2  7 12 17 22 27 32 37
      #  3  6 13 16 23 26 33 36
      #  4  5 14 15 24 25 34 35


      # map pins to should-be values
      if index <= 9:
        return index

      if index >= 12 and index  <= 21:
        return 31 -index  

      if index >= 24 and index  <= 33:
        return index - 4   

      if index > 3:
        return index - 8    

      else:
        print("index can't be ")

    def map_indices_for_printing(self,states):
        # States:
        #  0  9 10 19 20 29 30 39
        #  1  8 11 18 21 28 31 38
        #  2  7 12 17 22 27 32 37
        #  3  6 13 16 23 26 33 36
        #  4  5 14 15 24 25 34 35

        # Rearranged Print-Array Indices:
        #  0  1  2  3  4  5  6  7
        #  8  9 10 11 12 13 14 15
        # 16 17 18 19 20 21 22 23
        # 24 25 26 27 28 29 30 31
        # 32 33 34 35 36 37 38 39
        
        array = [0]*10*len(self.mprs)

        for i in range(len(array)):
            #gerade spalten
            if i%8 == 0: array[i] = states[int(i/8)]
            if i%8 == 2: array[i] = states[10+int(i/8)]
            if i%8 == 4: array[i] = states[20+int(i/8)]
            if i%8 == 6: array[i] = states[30+int(i/8)]

            #ungerade spalten
            if i%8 == 1: array[i] = states[9-int(i/8)]
            if i%8 == 3: array[i] = states[19-int(i/8)]
            if i%8 == 5: array[i] = states[29-int(i/8)]
            if i%8 == 7: array[i] = states[39-int(i/8)]            

        return array    

    def print_matrix(self,states):
        print_array = self.map_indices_for_printing(states)
        #print(print_array)
        print('----------------------------')
        line = " "
        for index in range(len(print_array)):
            if index%8 == 0:
                print(line)
                line = " "
            if print_array[index]==1:
                #print('1 at',index)
                line = line +"X "
            else:
                line = line +"- "
        print(line)  
    
    def get_touch_states(self):
        while self.end == False:
            currMPRS =-1 
            for mpr in self.mprs:
                currMPRS=currMPRS+1
                for i in range(12):
                    pinIndex = i+(12*currMPRS)
                    if pinIndex in self.connectedPins:
                      mappedPin = self.map_pins(pinIndex)
                      
                      #if mappedPin == 39:
                          #print('Pin',i,'Filt:',mpr.filtered_data(i),'Base:',mpr.baseline_data(i))
                      if mpr[i].value:
                          #Pin is being touched
                          if  self.states[mappedPin] == 0:
                            #Pin has not been touched in the last repetition of the loop --> touch has started
                            self.states[mappedPin] = 1
                            return(mappedPin,1)

                                
                      else:
                          #Pin is not being touched
                          if self.states[mappedPin] == 1:
                            #Pin has been touched in the last repetition of the loop --> touch has finished
                            time_up = datetime.datetime.now()
                            #avoid trigerring by trembles 
                            if (time_up-self.last_time_up[mappedPin]).total_seconds() > 0.06:
                                 self.states[mappedPin] = 0
                                 return (mappedPin,0)
                                
                            last_time_up[mappedPin] =   time_up                    

    def get_input(self):
        states = [0]*10*len(self.mprs)
        times_down =[""]*10*len(self.mprs)
        last_time_up = [datetime.datetime.now()]*10*len(self.mprs)
        
        while self.end == False:
            currMPRS =-1 
            for mpr in self.mprs:
                currMPRS=currMPRS+1
                for i in range(12):
                    pinIndex = i+(12*currMPRS)
                    if pinIndex in self.connectedPins:
                      mappedPin = self.map_pins(pinIndex)
                      
                      #if mappedPin == 39:
                          #print('Pin',i,'Filt:',mpr.filtered_data(i),'Base:',mpr.baseline_data(i))
                      if mpr[i].value:
                          #Pin is being touched
                          if  states[mappedPin] == 0:
                            #Pin has not been touched in the last repetition of the loop --> touch has started
                            times_down[mappedPin] = datetime.datetime.now()
                            #print("Input",mappedPin,"down!")
                            #print("Pin",i," mpr",currMPRS)
                            states[mappedPin] = 1
                            #self.print_matrix(states)
                                
                      else:
                          #Pin is not being touched
                          if times_down[mappedPin]!= "" and states[mappedPin] == 1:
                            time_up = datetime.datetime.now()
                            #avoid trigerring by trembles 
                            if (time_up-last_time_up[mappedPin]).total_seconds() > 0.06:
                                 #Pin has been touched in the last repetition of the loop --> touch has finished
                                 #print("Input",mappedPin,"up!")
                                 states[mappedPin] = 0
                                 #self.print_matrix(states)
                                 #print(mappedIndex, (time_up-times_down[mappedIndex]).total_seconds())
                                 return mappedPin, times_down[mappedPin], time_up
                                
                            last_time_up[mappedPin] =   time_up                           
                            
                    

#obj = mpr121()
#obj.get_input()
